import { __decorate } from 'tslib';
import { memoize, memoized } from '@compositive/commons-memoize';
import { inspect } from '@compositive/commons-util/debug';
import { Style, isColorHSL, isColorRGB, SxProcessorProvider, View as View$1, Text as Text$1, Image as Image$1, Input as Input$1, Upload as Upload$1 } from '@compositive/primitives';
export { Style as StaticStyle } from '@compositive/primitives';
import { HSL, sRGB, to, contrastWCAG21 } from 'colorjs.io/fn';
import { WeakKeyCache, ErrorCache, RecordCache, LRUCache } from '@compositive/commons-memoize/caches';
import { mapObject, enumerable, compactObject, freeze, pick, coalesce, titleize, asArray } from '@compositive/commons-util';
import { useMemo, createContext, useEffect, useContext, useState, useCallback } from 'react';
import { createPredicate, Equals, isDefined, equals, isInstanceOf, isOptional, isNumber, isObjectWithProperties, isEnum, isString } from '@compositive/commons-predicates';
import { jsx, jsxs } from 'react/jsx-runtime.js';
import { ChainEvaluator, Spec } from '@compositive/commons-spec';
import { every, Sequence } from '@compositive/commons-sequences';
import hsluv from 'hsluv';

const resolveThemedStyleProperties = memoize((themedStyleProperties, theme) => {
    const styleProps = typeof themedStyleProperties === "function"
        ? Reflect.apply(themedStyleProperties, null, [theme])
        : themedStyleProperties;
    return mapObject(styleProps, (value) => (typeof value === "function"
        ? Reflect.apply(value, null, [theme])
        : value));
}, { argCaches: (i) => (i < 2 ? new WeakKeyCache() : ErrorCache) });

const isConvertibleToStyle = (value) => {
    return (typeof value === "object" &&
        value != null &&
        typeof value.toStaticStyle === "function");
};

const createThemeSxProcessor = (theme) => (style) => {
    if (isConvertibleToStyle(style))
        return style.toStaticStyle(theme);
    if (style instanceof Style)
        return style;
    return resolveThemedStyleProperties(style, theme);
};

const byVariant = {
    caches: (i) => i === 0
        ? new RecordCache(new LRUCache({ limit: 16 }))
        : ErrorCache,
};
const byTheme = {
    caches: (i) => (i === 0 ? new WeakKeyCache() : ErrorCache),
};

class ThemedStyle {
    // TODO: Copying this static method here is not the best API design decision.
    // It would be better to find a better structure to separate "Style" manipulation from
    // the "SxProp" manipulation
    static concat(...sxProps) {
        return Style.concat(...sxProps);
    }
    #themedStyleProperties;
    constructor(themedStyleProperties) {
        this.#themedStyleProperties = themedStyleProperties;
    }
    toStaticStyle(theme) {
        return new Style(resolveThemedStyleProperties(this.#themedStyleProperties, theme));
    }
}
__decorate([
    memoized(byTheme)
], ThemedStyle.prototype, "toStaticStyle", null);

const useStaticStyle = (rules, deps) => useMemo(() => new Style(rules()), 
// eslint-disable-next-line react-hooks/exhaustive-deps
deps);

const useThemedStyle = (rules, deps) => useMemo(() => new ThemedStyle(rules()), 
// eslint-disable-next-line react-hooks/exhaustive-deps
deps);

const RRGGBBAA = /^#([\da-fA-F]{2})([\da-fA-F]{2})([\da-fA-F]{2})([\da-fA-F][\da-fA-F])?$/i;
const RGBA = /^#([\da-fA-F])([\da-fA-F])([\da-fA-F])([\da-fA-F])?$/i;
const isHexColorString = createPredicate((value) => typeof value === "string" && (RRGGBBAA.test(value) || RGBA.test(value)), "a hex color representation");
const REPEATED_RRGGBBAA = /#([\da-fA-F])\1([\da-fA-F])\2([\da-fA-F])\3(([\da-fA-F])\5)?$/i;
function parseHexColorString(color) {
    let match = RRGGBBAA.exec(color);
    if (match) {
        return {
            r: parseInt(match[1], 16),
            g: parseInt(match[2], 16),
            b: parseInt(match[3], 16),
            a: parseInt(match[4] ?? "ff", 16) / 255,
        };
    }
    match = RGBA.exec(color);
    if (match) {
        return {
            r: parseInt(match[1], 16) * 17,
            g: parseInt(match[2], 16) * 17,
            b: parseInt(match[3], 16) * 17,
            a: parseInt(match[4] ?? "f", 16) / 15,
        };
    }
    throw new TypeError(`Cannot parse color: '${color}'`);
}
function toHexColorString(rgb) {
    let color = "#" +
        [rgb.r, rgb.g, rgb.b]
            .map((channel) => channel.toString(16).padStart(2, "0"))
            .join("");
    const alpha = rgb.a ?? 1;
    if (alpha !== 1) {
        color += Math.round(alpha * 255)
            .toString(16)
            .padStart(2, "0");
    }
    const match = REPEATED_RRGGBBAA.exec(color);
    if (match) {
        const [, r, g, b, , a] = match;
        color = `#${r}${g}${b}${a ?? ""}`;
    }
    return color;
}

class Color {
    static fromBinary([r, g, b]) {
        return new Color({ r, g, b });
    }
    #colorObj;
    constructor(colorValue) {
        if (colorValue?.constructor === Color)
            return colorValue;
        if (isColorHSL(colorValue)) {
            const { h, s, l, a } = colorValue;
            this.#colorObj = { space: HSL, coords: [h, s, l], alpha: a ?? 1 };
        }
        else {
            let rgb;
            if (isHexColorString(colorValue))
                rgb = parseHexColorString(colorValue);
            else if (colorValue instanceof Color || isColorRGB(colorValue))
                rgb = colorValue;
            else
                throw new TypeError(`Unknown color value: ${inspect(colorValue)}`);
            this.#colorObj = {
                space: sRGB,
                coords: [rgb.r / 255, rgb.g / 255, rgb.b / 255],
                alpha: rgb.a ?? 1,
            };
        }
    }
    asForeground() {
        return new Style({ color: this });
    }
    asBackground() {
        return new Style({ backgroundColor: this });
    }
    toStaticStyle() {
        return this.asForeground();
    }
    get hex() {
        return toHexColorString(this);
    }
    get r() {
        return this.rgb[0];
    }
    get g() {
        return this.rgb[1];
    }
    get b() {
        return this.rgb[2];
    }
    get h() {
        return this.hsl[0];
    }
    get s() {
        return this.hsl[1];
    }
    get l() {
        return this.hsl[2];
    }
    get a() {
        return this.#colorObj.alpha ?? 1;
    }
    get rgb() {
        return to(this.#colorObj, sRGB).coords.map((c) => Math.round(c * 255));
    }
    get hsl() {
        return to(this.#colorObj, HSL).coords;
    }
    alpha(alpha) {
        const { r, g, b } = this;
        return new Color({ r, g, b, a: alpha });
    }
    toString() {
        return this.hex;
    }
    toBinary() {
        return new Uint8Array([this.r, this.g, this.b]);
    }
}
__decorate([
    memoized
], Color.prototype, "asForeground", null);
__decorate([
    memoized
], Color.prototype, "asBackground", null);
__decorate([
    memoized
], Color.prototype, "hex", null);
__decorate([
    memoized
], Color.prototype, "rgb", null);
__decorate([
    memoized
], Color.prototype, "hsl", null);

class Theme {
    get allFonts() {
        return Array.from(Object.values(this.typography).reduce((set, spec) => spec == null
            ? set
            : spec.allFonts.reduce((acc, font) => acc.add(font), set), new Set()));
    }
    async loadAllFonts() {
        await every(this.allFonts)
            .groupBy((font) => font.fontLoader)
            .mapAsync(([loader, fonts]) => loader.loadMany(fonts));
    }
    areFontsLoaded() {
        return this.allFonts.every((font) => font.isLoaded());
    }
}
__decorate([
    memoized
], Theme.prototype, "allFonts", null);

// FIXME: The previous type in the definition thunks for non-existing attributes should be "never" instead of "unknown"
// FIXME: Variant options should merge, not be replaced
/**
 * A {@link ComputedTheme} is a {@link Theme} implementation that allows for its values to be derived through the use of
 * pure functions and derivation rules.
 *
 * It's possible to create new themes from scratch, or through the extension of other themes,
 * by overriding only some of the values.
 */
class ComputedTheme extends Theme {
    static with(definition) {
        return new ComputedTheme([
            { definition, variants: {} },
        ]);
    }
    #chain;
    constructor(chain) {
        super();
        this.#chain = chain;
    }
    using(activeVariants) {
        return new ThemeSnapshot(this.#buildEvaluator(activeVariants), this, activeVariants);
    }
    [Equals](other) {
        return this === other;
    }
    with(definition) {
        return new ComputedTheme([
            ...this.#chain,
            { definition, variants: {} },
        ]);
    }
    when(variantName, options) {
        return new ComputedTheme([
            ...this.#chain,
            { definition: {}, variants: { [variantName]: options } },
        ]);
    }
    get surfaces() {
        return this.using({}).surfaces;
    }
    get typography() {
        return this.using({}).typography;
    }
    get metrics() {
        return this.using({}).metrics;
    }
    get palette() {
        return this.using({}).palette;
    }
    get colorPicks() {
        return this.using({}).colorPicks;
    }
    get variants() {
        const options = {};
        for (const { variants } of this.#chain) {
            for (const [name, definitions] of Object.entries(variants)) {
                options[name] = [
                    ...new Set([...(options[name] ?? []), ...Object.keys(definitions)]),
                ];
            }
        }
        return options;
    }
    #buildEvaluator(activeVariants = {}) {
        const sortedActiveVariantKeys = Object.keys(activeVariants).sort();
        const definitions = [];
        for (const { definition, variants } of this.#chain) {
            definitions.push(definition);
            for (const variantName of sortedActiveVariantKeys) {
                const activeOption = activeVariants[variantName];
                if (activeOption == null)
                    continue;
                const variantDefinition = variants[variantName]?.[activeOption];
                if (variantDefinition != null)
                    definitions.push(variantDefinition);
            }
        }
        return ChainEvaluator.create(1, definitions);
    }
}
__decorate([
    memoized(byVariant)
], ComputedTheme.prototype, "using", null);
__decorate([
    enumerable
], ComputedTheme.prototype, "surfaces", null);
__decorate([
    enumerable
], ComputedTheme.prototype, "typography", null);
__decorate([
    enumerable
], ComputedTheme.prototype, "metrics", null);
__decorate([
    enumerable
], ComputedTheme.prototype, "palette", null);
__decorate([
    enumerable
], ComputedTheme.prototype, "colorPicks", null);
__decorate([
    memoized
], ComputedTheme.prototype, "variants", null);
class ThemeSnapshot extends Theme {
    #evaluator;
    #origin;
    #activeVariants;
    constructor(evaluator, origin, activeVariants) {
        super();
        this.#evaluator = evaluator;
        this.#origin = origin;
        this.#activeVariants = activeVariants;
    }
    using(activeVariants) {
        return this.#origin.using({ ...this.#activeVariants, ...activeVariants });
    }
    get surfaces() {
        return this.#evaluator.surfaces;
    }
    get typography() {
        return this.#evaluator.typography;
    }
    get metrics() {
        return this.#evaluator.metrics;
    }
    get palette() {
        return this.#evaluator.palette;
    }
    get colorPicks() {
        return this.#evaluator.colorPicks;
    }
    get variants() {
        return this.#origin.variants;
    }
}
__decorate([
    enumerable
], ThemeSnapshot.prototype, "surfaces", null);
__decorate([
    enumerable
], ThemeSnapshot.prototype, "typography", null);
__decorate([
    enumerable
], ThemeSnapshot.prototype, "metrics", null);
__decorate([
    enumerable
], ThemeSnapshot.prototype, "palette", null);
__decorate([
    enumerable
], ThemeSnapshot.prototype, "colorPicks", null);

const units = (multiplier = 1) => (theme) => `${theme.metrics.grid * multiplier}px`;
const u = units;

const ThemeContext = createContext(undefined);
const THEME_NOT_PROVIDED_ERROR = "No theme found in the component hierachy. Please wrap your components in a ThemeProvider element.";

const ThemeConsumer = ({ children, }) => {
    return (jsx(ThemeContext.Consumer, { children: (theme) => children(isDefined.validate(theme, THEME_NOT_PROVIDED_ERROR)) }));
};

const ThemeProvider = ({ theme, children, ...variants }) => {
    const themeWithVariants = useMemo(() => (Object.keys(variants).length === 0 ? theme : theme.using(variants)), [theme, variants]);
    useEffect(() => {
        themeWithVariants
            .loadAllFonts()
            .catch((error) => console.error("Error while trying to load the theme fonts:", error));
    }, [themeWithVariants]);
    const themeSxProcessor = useMemo(() => createThemeSxProcessor(themeWithVariants), [themeWithVariants]);
    return (jsx(ThemeContext.Provider, { value: themeWithVariants, children: jsx(SxProcessorProvider, { value: themeSxProcessor, children: children }) }));
};

const useTheme = () => {
    const theme = useContext(ThemeContext);
    isDefined.assert(theme, THEME_NOT_PROVIDED_ERROR);
    return theme;
};

class Shade extends Color {
    colorScale;
    index;
    constructor(colorScale, index, colorValue) {
        super(colorValue);
        this.colorScale = colorScale;
        this.index = index;
    }
    transposeOn(otherScale) {
        return otherScale.get(this.index);
    }
    backward(steps = 1, outOfBoundariesMethod = "none") {
        return this.colorScale.get(this.index - steps, outOfBoundariesMethod);
    }
    frontward(steps = 1, outOfBoundariesMethod = "none") {
        return this.colorScale.get(this.index + steps, outOfBoundariesMethod);
    }
    [Equals](other) {
        const ShadeClass = this.constructor;
        return (other instanceof ShadeClass &&
            other.hex === this.hex &&
            other.index === this.index);
    }
}

const DEFAULT_PROPS = {
    foreground: "foreground",
    background: "background",
    palette: "neutral",
};
const ColorContext = createContext(DEFAULT_PROPS);
const ColorProvider = (props) => {
    const previousInput = useContext(ColorContext);
    const { foreground, background, palette, children } = {
        ...previousInput,
        ...compactObject(props),
    };
    return (jsx(ColorContext.Provider, { value: { foreground, background, palette }, children: children }));
};
const ColorConsumer = ({ children, }) => {
    const values = useCurrentColors();
    return children(values);
};
const resolveShade = (theme, palette, pick, background) => {
    if (pick instanceof Shade) {
        return pick;
    }
    if (typeof pick === "string") {
        return resolveShade(theme, palette, theme.colorPicks[pick], background);
    }
    return pick.pickColor(palette, background);
};
const useCurrentColors = () => {
    const refs = useContext(ColorContext);
    const theme = useTheme();
    const palette = theme.palette[refs.palette];
    const background = resolveShade(theme, palette, refs.background);
    const foreground = resolveShade(theme, palette, refs.foreground, background);
    return { palette, background, foreground };
};
const useCurrentForeground = () => useCurrentColors().foreground;
const useCurrentBackground = () => useCurrentColors().background;
const useCurrentPalette = () => useCurrentColors().palette;

const ColorPicker = {
    create(fn) {
        return { pickColor: fn };
    },
};

const rgb = (r, g, b) => new Color({ r, g, b });
const rgba = (r, g, b, a) => new Color({ r, g, b, a });
const hsl = (h, s, l) => new Color({ h, s, l });
const hsla = (h, s, l, a) => new Color({ h, s, l, a });
const hex = (color) => {
    let hexValue = color;
    if (hexValue.startsWith("#"))
        hexValue = hexValue.slice(1);
    if (hexValue.length === 3)
        return new Color({
            r: parseInt(hexValue.charAt(0), 16) * 0x11,
            g: parseInt(hexValue.charAt(1), 16) * 0x11,
            b: parseInt(hexValue.charAt(2), 16) * 0x11,
        });
    if (hexValue.length === 6)
        return new Color({
            r: parseInt(hexValue.slice(0, 2), 16),
            g: parseInt(hexValue.slice(2, 4), 16),
            b: parseInt(hexValue.slice(4, 6), 16),
        });
    throw new TypeError(`Unknown color format: ${color}`);
};

const transparent = rgba(0, 0, 0, 0);
// Red colors
const mediumVioletRed = rgb(199, 21, 133);
const deepPink = rgb(255, 20, 147);
const paleVioletRed = rgb(219, 112, 147);
const hotPink = rgb(255, 105, 180);
const lightPink = rgb(255, 182, 193);
const pink = rgb(255, 192, 203);
const darkRed = rgb(139, 0, 0);
const red = rgb(255, 0, 0);
const firebrick = rgb(178, 34, 34);
const crimson = rgb(220, 20, 60);
const indianRed = rgb(205, 92, 92);
const lightCoral = rgb(240, 128, 128);
const salmon = rgb(250, 128, 114);
const darkSalmon = rgb(233, 150, 122);
const lightSalmon = rgb(255, 160, 122);
// Orange colors
const orangeRed = rgb(255, 69, 0);
const tomato = rgb(255, 99, 71);
const darkOrange = rgb(255, 140, 0);
const coral = rgb(255, 127, 80);
const orange = rgb(255, 165, 0);
// Yellow colors
const darkKhaki = rgb(189, 183, 107);
const gold = rgb(255, 215, 0);
const khaki = rgb(240, 230, 140);
const peachPuff = rgb(255, 218, 185);
const yellow = rgb(255, 255, 0);
const paleGoldenrod = rgb(238, 232, 170);
const moccasin = rgb(255, 228, 181);
const papayaWhip = rgb(255, 239, 213);
const lightGoldenrodYellow = rgb(250, 250, 210);
const lemonChiffon = rgb(255, 250, 205);
const lightYellow = rgb(255, 255, 224);
// Brown colors
const maroon = rgb(128, 0, 0);
const brown = rgb(165, 42, 42);
const saddleBrown = rgb(139, 69, 19);
const sienna = rgb(160, 82, 45);
const chocolate = rgb(210, 105, 30);
const darkGoldenrod = rgb(184, 134, 11);
const peru = rgb(205, 133, 63);
const rosyBrown = rgb(188, 143, 143);
const goldenrod = rgb(218, 165, 32);
const sandyBrown = rgb(244, 164, 96);
const tan = rgb(210, 180, 140);
const burlywood = rgb(222, 184, 135);
const wheat = rgb(245, 222, 179);
const navajoWhite = rgb(255, 222, 173);
const bisque = rgb(255, 228, 196);
const blanchedAlmond = rgb(255, 235, 205);
const cornsilk = rgb(255, 248, 220);
// Green colors
const darkGreen = rgb(0, 100, 0);
const green = rgb(0, 128, 0);
const darkOliveGreen = rgb(85, 107, 47);
const forestGreen = rgb(34, 139, 34);
const seaGreen = rgb(46, 139, 87);
const olive = rgb(128, 128, 0);
const oliveDrab = rgb(107, 142, 35);
const mediumSeaGreen = rgb(60, 179, 113);
const limeGreen = rgb(50, 205, 50);
const lime = rgb(0, 255, 0);
const springGreen = rgb(0, 255, 127);
const mediumSpringGreen = rgb(0, 250, 154);
const darkSeaGreen = rgb(143, 188, 143);
const mediumAquamarine = rgb(102, 205, 170);
const yellowGreen = rgb(154, 205, 50);
const lawnGreen = rgb(124, 252, 0);
const chartreuse = rgb(127, 255, 0);
const lightGreen = rgb(144, 238, 144);
const greenYellow = rgb(173, 255, 47);
const paleGreen = rgb(152, 251, 152);
// Cyan colors
const teal = rgb(0, 128, 128);
const darkCyan = rgb(0, 139, 139);
const lightSeaGreen = rgb(32, 178, 170);
const cadetBlue = rgb(95, 158, 160);
const darkTurquoise = rgb(0, 206, 209);
const mediumTurquoise = rgb(72, 209, 204);
const turquoise = rgb(64, 224, 208);
const aqua = rgb(0, 255, 255);
const cyan = rgb(0, 255, 255);
const aquamarine = rgb(127, 255, 212);
const paleTurquoise = rgb(175, 238, 238);
const lightCyan = rgb(224, 255, 255);
// Blue colors
const navy = rgb(0, 0, 128);
const darkBlue = rgb(0, 0, 139);
const mediumBlue = rgb(0, 0, 205);
const blue = rgb(0, 0, 255);
const midnightBlue = rgb(25, 25, 112);
const royalBlue = rgb(65, 105, 225);
const steelBlue = rgb(70, 130, 180);
const dodgerBlue = rgb(30, 144, 255);
const deepSkyBlue = rgb(0, 191, 255);
const cornflowerBlue = rgb(100, 149, 237);
const skyBlue = rgb(135, 206, 235);
const lightSkyBlue = rgb(135, 206, 250);
const lightSteelBlue = rgb(176, 196, 222);
const lightBlue = rgb(173, 216, 230);
const powderBlue = rgb(176, 224, 230);
// Purple, violet, and magenta colors
const indigo = rgb(75, 0, 130);
const purple = rgb(128, 0, 128);
const darkMagenta = rgb(139, 0, 139);
const darkViolet = rgb(148, 0, 211);
const darkSlateBlue = rgb(72, 61, 139);
const blueViolet = rgb(138, 43, 226);
const darkOrchid = rgb(153, 50, 204);
const fuchsia = rgb(255, 0, 255);
const magenta = rgb(255, 0, 255);
const slateBlue = rgb(106, 90, 205);
const mediumSlateBlue = rgb(123, 104, 238);
const mediumOrchid = rgb(186, 85, 211);
const mediumPurple = rgb(147, 112, 219);
const orchid = rgb(218, 112, 214);
const violet = rgb(238, 130, 238);
const plum = rgb(221, 160, 221);
const thistle = rgb(216, 191, 216);
const lavender = rgb(230, 230, 250);
// White colors
const mistyRose = rgb(255, 228, 225);
const antiqueWhite = rgb(250, 235, 215);
const linen = rgb(250, 240, 230);
const beige = rgb(245, 245, 220);
const whiteSmoke = rgb(245, 245, 245);
const lavenderBlush = rgb(255, 240, 245);
const oldLace = rgb(253, 245, 230);
const aliceBlue = rgb(240, 248, 255);
const seashell = rgb(255, 245, 238);
const ghostWhite = rgb(248, 248, 255);
const honeydew = rgb(240, 255, 240);
const floralWhite = rgb(255, 250, 240);
const azure = rgb(240, 255, 255);
const mintCream = rgb(245, 255, 250);
const snow = rgb(255, 250, 250);
const ivory = rgb(255, 255, 240);
const white = rgb(255, 255, 255);
// Gray and black colors
const black = rgb(0, 0, 0);
const darkSlateGray = rgb(47, 79, 79);
const dimGray = rgb(105, 105, 105);
const slateGray = rgb(112, 128, 144);
const gray = rgb(128, 128, 128);
const lightSlateGray = rgb(119, 136, 153);
const darkGray = rgb(169, 169, 169);
const silver = rgb(192, 192, 192);
const lightGray = rgb(211, 211, 211);
const gainsboro = rgb(220, 220, 220);

class ColorScale extends Sequence {
    constructor() {
        super(() => this.shades[Symbol.iterator]());
    }
    get(key, outOfBoundsMethod = "none") {
        switch (outOfBoundsMethod) {
            case "wrap": {
                if (key >= 0 && key < this.length)
                    return this.shades[key];
                const lastIndex = this.length - 1;
                const absKey = Math.abs(key);
                const rounds = Math.floor(absKey / this.length);
                const offset = absKey % lastIndex;
                return this.shades[(rounds % 2 === 0 ? lastIndex : 0) - offset * (-1) ** rounds];
            }
            case "clamp": {
                return this.shades[Math.min(this.length - 1, Math.max(0, key))];
            }
            default: {
                return this.shades[key];
            }
        }
    }
    has(key) {
        return key >= 0 && key < this.length;
    }
    get length() {
        return this.shades.length;
    }
    get backmost() {
        return this.shades[0];
    }
    get midpoint() {
        const midIndex = Math.min(Math.round(this.shades.length / 2), this.shades.length - 1);
        return this.shades[midIndex];
    }
    get frontmost() {
        return this.shades[this.shades.length - 1];
    }
    filter(cb) {
        return new FixedColorScale(super.filter(cb));
    }
    [Equals](other) {
        const ColorScaleClass = this.constructor;
        return (other instanceof ColorScaleClass &&
            other.constructor === ColorScaleClass &&
            other.length === this.length &&
            equals([...other].map((t) => t.hex), [...this].map((t) => t.hex)));
    }
    buildNextWithSameElementKind(source) {
        return new FixedColorScale(source);
    }
}
class FixedColorScale extends ColorScale {
    shades;
    constructor(colors) {
        super();
        this.shades = every(colors)
            .toArray()
            .map((c, i) => new Shade(this, i, c));
    }
    reversed() {
        return new FixedColorScale([...this.shades].reverse());
    }
}

var ContrastLevel;
(function (ContrastLevel) {
    /** Less than WCAG 3:1 contrast ratio */
    ContrastLevel[ContrastLevel["INSUFFICIENT"] = 0] = "INSUFFICIENT";
    /** Equivalent to WCAG A (3..4.5:1) level */
    ContrastLevel[ContrastLevel["MINIMUM"] = 1] = "MINIMUM";
    /** Equivalent to WCAG AA (4.5..7:1) level */
    ContrastLevel[ContrastLevel["GOOD"] = 2] = "GOOD";
    /** Equivalent to WCAG AAA (\>= 7:1) level */
    ContrastLevel[ContrastLevel["HIGH"] = 3] = "HIGH";
})(ContrastLevel || (ContrastLevel = {}));

const getWCAGContrast = (background, foreground) => {
    const value = contrastWCAG21({
        space: sRGB,
        coords: background.rgb.map((c) => c / 255),
    }, {
        space: sRGB,
        coords: foreground.rgb.map((c) => c / 255),
    });
    return {
        get value() {
            return value;
        },
        toString() {
            const roundedValue = Math.round(value * 100) / 100;
            const level = this.level;
            let levelDescription;
            switch (level) {
                case ContrastLevel.HIGH: {
                    levelDescription = " AAA";
                    break;
                }
                case ContrastLevel.GOOD: {
                    levelDescription = " AA";
                    break;
                }
                case ContrastLevel.MINIMUM: {
                    levelDescription = " A";
                    break;
                }
                default: {
                    levelDescription = "";
                }
            }
            return `${roundedValue}${levelDescription}`;
        },
        get level() {
            if (value >= 7)
                return ContrastLevel.HIGH;
            if (value >= 4.5)
                return ContrastLevel.GOOD;
            if (value >= 3)
                return ContrastLevel.MINIMUM;
            return ContrastLevel.INSUFFICIENT;
        },
    };
};

/** No easing, no acceleration */
const linear = (t) => t;
/** Accelerating from zero velocity */
const easeInQuad = (t) => t * t;
/** Decelerating to zero velocity */
const easeOutQuad = (t) => t * (2 - t);
/** Acceleration until halfway, then deceleration */
const easeInOutQuad = (t) => t < 0.5 ? 2 * t * t : -1 + (4 - 2 * t) * t;
/** Accelerating from zero velocity */
const easeInCubic = (t) => t ** 3;
/** Decelerating to zero velocity */
const easeOutCubic = (t) => (t - 1) ** 3 + 1;
/** Acceleration until halfway, then deceleration */
const easeInOutCubic = (t) => t < 0.5 ? 4 * t ** 3 : (t - 1) * (2 * t - 2) * (2 * t - 2) + 1;
/** Accelerating from zero velocity */
const easeInQuart = (t) => t ** 4;
/** Decelerating to zero velocity */
const easeOutQuart = (t) => 1 - (t - 1) ** 4;
/** Acceleration until halfway, then deceleration */
const easeInOutQuart = (t) => t < 0.5 ? 8 * t ** 4 : 1 - 8 * (t - 1) ** 4;
/** Accelerating from zero velocity */
const easeInQuint = (t) => t ** 5;
/** Decelerating to zero velocity */
const easeOutQuint = (t) => 1 + (t - 1) ** 5;
/** Acceleration until halfway, then deceleration */
const easeInOutQuint = (t) => t < 0.5 ? 16 * t ** 5 : 1 + 16 * (t - 1) ** 5;
/** Hyperbolic tangent, normalized to be bounded in the interval [0, 1] */
const tanh = (t) => Math.tanh(t * 8 - 4) / 2 + 0.5;

const { hexToHsluv, hsluvToHex } = hsluv;
class UniformLightnessColorScale extends ColorScale {
    baseColor;
    lightnessMin;
    lightnessMax;
    easing;
    steps;
    constructor(base, steps = 10, options = {}) {
        super();
        this.steps = steps;
        this.baseColor = new Color(base);
        this.lightnessMin = options?.lightnessMin ?? 1;
        this.lightnessMax = options?.lightnessMax ?? 99;
        this.easing = options?.easing ?? linear;
    }
    get length() {
        return this.steps;
    }
    get shades() {
        const [hue, saturation] = hexToHsluv(this.baseColor.hex);
        const step = 1 / (this.steps - 1);
        const lightnesses = [];
        for (let i = 0; i < this.steps; ++i)
            lightnesses[i] =
                this.lightnessMax +
                    (this.lightnessMin - this.lightnessMax) * this.easing(i * step);
        return lightnesses.map((lightness, i) => new Shade(this, i, hsluvToHex([hue, saturation, lightness])));
    }
    reversed() {
        return new UniformLightnessColorScale(this.baseColor, this.steps, {
            lightnessMax: this.lightnessMin,
            lightnessMin: this.lightnessMax,
            easing: this.easing,
        });
    }
}
__decorate([
    memoized
], UniformLightnessColorScale.prototype, "shades", null);

const usePalette = (name) => {
    const theme = useTheme();
    return theme.palette[name];
};

const View = View$1;
const Text = Text$1;
const Image = Image$1;
const Input = Input$1;
const Upload = Upload$1;

class SolidFill extends Spec({ shade: isInstanceOf(Shade) }) {
    type = "background";
    process(parts, chain) {
        return [parts.addInnerStyles(this._getStyle()), chain];
    }
    _getStyle() {
        return new Style({ backgroundColor: this.shade });
    }
}
__decorate([
    memoized
], SolidFill.prototype, "_getStyle", null);

class SolidBorder extends Spec({
    width: isOptional(isNumber),
    shade: isInstanceOf(Shade),
}) {
    type = "border";
    process(parts, chain) {
        return [parts.addInnerStyles(this._getStyle()), chain];
    }
    _getStyle() {
        return new Style({
            borderWidth: this.width ?? 1,
            borderColor: this.shade,
        });
    }
}
__decorate([
    memoized
], SolidBorder.prototype, "_getStyle", null);

const isRoundedCornersAttributes = isObjectWithProperties({
    radius: isOptional(isNumber),
    topLeft: isOptional(isNumber),
    topRight: isOptional(isNumber),
    bottomLeft: isOptional(isNumber),
    bottomRight: isOptional(isNumber),
});
class RoundedCorners extends Spec(isRoundedCornersAttributes) {
    type = "corners";
    constructor(radiusOrAttribs, previousSpec) {
        super(typeof radiusOrAttribs === "number"
            ? { radius: radiusOrAttribs }
            : radiusOrAttribs, previousSpec);
    }
    process(parts, chain) {
        return [parts.addInnerStyles(this._getStyle()), chain];
    }
    // FIXME: Use the other attributes
    _getStyle() {
        return new Style({ borderRadius: this.radius });
    }
}
__decorate([
    memoized
], RoundedCorners.prototype, "_getStyle", null);

const getShadowStyle = (elevation, tone) => {
    const blurRadius = elevation === 1 ? 3 : elevation * 2;
    const ambientShadow = {
        x: 0,
        y: elevation,
        blurRadius,
        color: tone.alpha((elevation + 10 + elevation / 9.38) / 100),
    };
    const directShadow = {
        x: 0,
        y: elevation < 10
            ? elevation % 2 === 0
                ? elevation - (elevation / 2 - 1)
                : elevation - (elevation - 1) / 2
            : elevation - 4,
        blurRadius,
        color: tone.alpha((24 - Math.round(elevation / 10)) / 100),
    };
    return new Style({ boxShadow: [ambientShadow, directShadow] });
};
class DropShadow extends Spec({
    elevation: isNumber,
    tone: isInstanceOf(Shade),
}) {
    type = "shadow";
    process(parts, chain) {
        const { elevation = 0, tone = black } = this;
        return [
            elevation !== 0
                ? parts.addInnerStyles(getShadowStyle(elevation, tone))
                : parts,
            chain,
        ];
    }
}

class Rectangle extends Spec({ padding: isOptional(isNumber) }) {
    type = "shape";
    constructor(attribs = {}, previousSpec) {
        super(attribs, previousSpec);
    }
    process(parts, chain) {
        return [parts.addInnerStyles(this._getStyle()), chain];
    }
    _getStyle() {
        return new Style({ padding: `${this.padding ?? 0}px` });
    }
}
__decorate([
    memoized
], Rectangle.prototype, "_getStyle", null);

class SurfaceParts {
    #prependedComponents = [];
    #appendedComponents = [];
    #innerStyles;
    #outerStyles;
    get prependedComponents() {
        return this.#prependedComponents;
    }
    prependComponent(component) {
        const next = this.#clone();
        next.#prependedComponents = [component, ...this.#prependedComponents];
        return next;
    }
    get appendedComponents() {
        return this.#appendedComponents;
    }
    appendComponent(component) {
        const next = this.#clone();
        next.#appendedComponents = [...this.#appendedComponents, component];
        return next;
    }
    get innerStyles() {
        return this.#innerStyles;
    }
    addInnerStyles(styles) {
        const next = this.#clone();
        next.#innerStyles = ThemedStyle.concat(this.#innerStyles, styles);
        return next;
    }
    get outerStyles() {
        return this.#outerStyles;
    }
    addOuterStyles(styles) {
        const next = this.#clone();
        next.#outerStyles = ThemedStyle.concat(this.#outerStyles, styles);
        return next;
    }
    #clone() {
        const next = new SurfaceParts();
        next.#prependedComponents = this.#prependedComponents;
        next.#appendedComponents = this.#appendedComponents;
        next.#innerStyles = this.#innerStyles;
        next.#outerStyles = this.#outerStyles;
        return next;
    }
}

const ELEMENT_NAMES = freeze([
    "shape",
    "corners",
    "border",
    "background",
    "shadow",
]);
const ATTRIBUTE_NAMES = freeze([...ELEMENT_NAMES, "foreground"]);
const innerViewBaseStyle = new Style({ flex: 1 });
class SurfaceSpec extends Spec(ATTRIBUTE_NAMES) {
    toComponent() {
        const variantPropKeys = Object.keys(this.variants);
        const hasStateVariant = this.hasVariant("state");
        const SurfaceComponent = ({ children, as, sx, innerSx, onHoverStart, onHoverEnd, onPress, onPressIn, onPressOut, state, ...props }) => {
            const [internalState, setInternalState] = useState(state ?? "default");
            const onHoverStartCallback = useCallback((e) => {
                if (state !== "disabled") {
                    if (hasStateVariant)
                        setInternalState("hovered");
                    onHoverStart?.(e);
                }
            }, [state, onHoverStart, setInternalState]);
            const onHoverEndCallback = useCallback((e) => {
                if (state !== "disabled") {
                    if (hasStateVariant)
                        setInternalState("default");
                    onHoverEnd?.(e);
                }
            }, [state, onHoverEnd, setInternalState]);
            const onPressInCallback = useCallback((e) => {
                if (state !== "disabled") {
                    if (hasStateVariant)
                        setInternalState("pressed");
                    onPressIn?.(e);
                }
            }, [state, onPressIn, setInternalState]);
            const onPressOutCallback = useCallback((e) => {
                if (state !== "disabled") {
                    if (hasStateVariant)
                        setInternalState("hovered");
                    onPressOut?.(e);
                }
            }, [state, onPressOut, setInternalState]);
            const propsWithDefaults = {
                ...props,
                state: state ?? internalState,
            };
            const variantProps = pick(propsWithDefaults, variantPropKeys);
            const { parts, foreground } = this._getSurfaceParts(variantProps);
            return (jsxs(View, { sx: ThemedStyle.concat(parts.outerStyles, sx), as: as, onHoverStart: onHoverStartCallback, onHoverEnd: onHoverEndCallback, onPress: onPress, onPressIn: onPressInCallback, onPressOut: onPressOutCallback, children: [parts.prependedComponents, jsx(View, { sx: ThemedStyle.concat(innerViewBaseStyle, parts.innerStyles, innerSx), children: jsx(ColorProvider, { foreground: foreground, children: children }) }), parts.appendedComponents] }));
        };
        return SurfaceComponent;
    }
    _getSurfaceParts(activeVariants) {
        const { foreground, ...attributes } = this.using(activeVariants).attributes;
        let parts = new SurfaceParts();
        let chain = ELEMENT_NAMES.map((name) => attributes[name]).filter(isDefined.test);
        while (chain.length > 0) {
            const element = chain.shift();
            if (element != null)
                [parts, chain] = element.process(parts, chain);
        }
        return { parts, foreground };
    }
}
__decorate([
    memoized
], SurfaceSpec.prototype, "toComponent", null);
__decorate([
    memoized(byVariant)
], SurfaceSpec.prototype, "_getSurfaceParts", null);

function useSurface(names) {
    const { surfaces } = useTheme();
    const Component = coalesce(...names.map((name) => surfaces[name]))?.toComponent();
    isDefined.assert(Component, () => `Can't find a surface definition for this component in the theme. Looked up keys: ${names
        .map((n) => n.toString())
        .join(", ")}.`);
    return Component;
}

var FontStyle;
(function (FontStyle) {
    FontStyle["Normal"] = "normal";
    FontStyle["Italic"] = "italic";
    FontStyle["Oblique"] = "oblique";
})(FontStyle || (FontStyle = {}));

class FontWeight {
    static Thin = new FontWeight("Thin", 100);
    static ExtraLight = new FontWeight("ExtraLight", 200);
    static Light = new FontWeight("Light", 300);
    static Regular = new FontWeight("Regular", 400);
    static Medium = new FontWeight("Medium", 500);
    static SemiBold = new FontWeight("SemiBold", 600);
    static Bold = new FontWeight("Bold", 700);
    static ExtraBold = new FontWeight("ExtraBold", 800);
    static Black = new FontWeight("Black", 900);
    static for(weight) {
        if (weight > 0 && weight < 10)
            return ORDERED_WEIGHTS[Math.round(weight) - 1];
        if (weight >= 100 && weight <= 900)
            return ORDERED_WEIGHTS[Math.round(weight / 100) - 1];
        return FontWeight.Regular;
    }
    value;
    name;
    #index;
    constructor(name, value) {
        this.name = name;
        this.value = value;
    }
    get index() {
        this.#index ??= ORDERED_WEIGHTS.indexOf(this);
        return this.#index;
    }
    get bolder() {
        return this.index === ORDERED_WEIGHTS.length - 1
            ? void 0
            : ORDERED_WEIGHTS[this.index + 1];
    }
    get lighter() {
        return this.index === 0 ? void 0 : ORDERED_WEIGHTS[this.index + 1];
    }
    get fallbacks() {
        return ORDERED_WEIGHTS.slice(this.index);
    }
    isLighterThan(other) {
        return this.value < other.value;
    }
    isBolderThan(other) {
        return this.value > other.value;
    }
    toString() {
        return this.name;
    }
}
const ORDERED_WEIGHTS = [
    FontWeight.Thin,
    FontWeight.ExtraLight,
    FontWeight.Light,
    FontWeight.Regular,
    FontWeight.Medium,
    FontWeight.SemiBold,
    FontWeight.Bold,
    FontWeight.ExtraBold,
    FontWeight.Black,
];

const fontDesignationMap = new Map();
class FontDesignation {
    static fromFvd(fvd) {
        const style = {
            i: FontStyle.Italic,
            n: FontStyle.Normal,
            o: FontStyle.Oblique,
        }[fvd.charAt(0)] ?? FontStyle.Normal;
        const weight = FontWeight.for(Number.parseInt(fvd.charAt(1)));
        return this.for(weight, style);
    }
    static for(weight, style) {
        const key = toKey(weight, style);
        if (!fontDesignationMap.has(key)) {
            fontDesignationMap.set(key, new FontDesignation(weight, style));
        }
        return fontDesignationMap.get(key);
    }
    weight;
    style;
    constructor(weight, style) {
        this.weight = weight;
        this.style = style;
    }
    get fallbacks() {
        return this.weight.fallbacks.flatMap((weight) => fontStyleAlternatives(this.style).map((style) => FontDesignation.for(weight, style)));
    }
    get fvd() {
        const weight = this.weight.value / 100;
        const style = {
            [FontStyle.Normal]: "n",
            [FontStyle.Italic]: "i",
            [FontStyle.Oblique]: "o",
        }[this.style];
        return `${style}${weight}`;
    }
    get googleFontsDescription() {
        const weight = this.weight.value;
        const style = {
            [FontStyle.Normal]: "",
            [FontStyle.Italic]: "i",
            [FontStyle.Oblique]: "o",
        }[this.style];
        return `${weight}${style}`;
    }
    toString() {
        return toKey(this.weight, this.style);
    }
    toHumanString() {
        return titleize(this.toString());
    }
}
const toKey = (weight, style) => weight.name + (style === FontStyle.Normal ? "" : titleize(style));
const fontStyleAlternatives = (style) => {
    switch (style) {
        case FontStyle.Normal: {
            return [FontStyle.Normal];
        }
        case FontStyle.Italic: {
            return [FontStyle.Italic, FontStyle.Oblique];
        }
        case FontStyle.Oblique: {
            return [FontStyle.Oblique, FontStyle.Italic];
        }
    }
};

var TextTransform;
(function (TextTransform) {
    TextTransform["None"] = "none";
    TextTransform["Uppercase"] = "uppercase";
    TextTransform["Lowercase"] = "lowercase";
    TextTransform["Capitalize"] = "capitalize";
})(TextTransform || (TextTransform = {}));

class Font {
    family;
    weight;
    style;
    fontLoader;
    constructor(family, weight, style, fontLoader) {
        this.family = family;
        this.weight = weight;
        this.style = style;
        this.fontLoader = fontLoader;
    }
    withSize(size) {
        return new FontSpec({ font: this, size });
    }
    with(attributeNameOrValues, value) {
        if (isString(attributeNameOrValues)) {
            isDefined.assert(value);
            return this.withSize(value);
        }
        else {
            return new FontSpec({ ...attributeNameOrValues, font: this });
        }
    }
    get designation() {
        return FontDesignation.for(this.weight, this.style);
    }
    get bolder() {
        const bolderWeight = this.weight.bolder;
        return bolderWeight != null
            ? this.family.getClosestFont(bolderWeight, this.style) ?? this
            : this;
    }
    get lighter() {
        const lighterWeight = this.weight.lighter;
        return lighterWeight != null
            ? this.family.getClosestFont(lighterWeight, this.style) ?? this
            : this;
    }
    get italic() {
        // TODO: Should fallback to a generic font?
        return this.family.getClosestFont(this.weight, FontStyle.Italic) ?? this;
    }
    get normal() {
        // TODO: Should fallback to a generic font?
        return this.family.getClosestFont(this.weight, FontStyle.Normal) ?? this;
    }
    async load() {
        return this.fontLoader.load(this);
    }
    isLoaded() {
        return this.fontLoader.isLoaded(this);
    }
    toString() {
        return `${this.family.name} ${this.designation.toHumanString()}`;
    }
}
class FontSpec extends Spec({
    size: isNumber,
    lineHeight: isOptional(isNumber),
    font: isInstanceOf(Font),
    letterSpacing: isOptional(isNumber),
    textTransform: isOptional(isEnum(TextTransform)),
}) {
    get allFonts() {
        const variantDefiner = this.variantDefiner;
        const combinations = variantDefiner.getCombinations();
        const fontSet = combinations.reduce((set, activeVariants) => set.add(this.using(activeVariants).font), new Set());
        return [...fontSet];
    }
    toStaticStyle() {
        const { size, letterSpacing, font, lineHeight, textTransform } = this;
        return new Style(compactObject({
            fontSize: Math.round(size),
            letterSpacing,
            fontFamily: font.family.toString(),
            fontWeight: font.weight.value,
            fontStyle: font.style,
            lineHeight,
            textTransform,
        }));
    }
}
__decorate([
    memoized
], FontSpec.prototype, "toStaticStyle", null);

var FontType;
(function (FontType) {
    FontType["Cursive"] = "cursive";
    FontType["Fantasy"] = "fantasy";
    FontType["Monospace"] = "monospace";
    FontType["SansSerif"] = "sans-serif";
    FontType["Serif"] = "serif";
})(FontType || (FontType = {}));
class FontFamily {
    name;
    fallback;
    fontType;
    #fontMap = new Map();
    constructor(name, fontType, fallback = []) {
        this.name = name;
        this.fontType = fontType;
        this.fallback = fallback;
    }
    withFont(weight, style) {
        const designation = FontDesignation.for(weight, style);
        const font = new Font(this, weight, style, this.fontLoader);
        this.#fontMap.set(designation, font);
        Object.defineProperty(this, designation.toString(), { get: () => font });
        return this;
    }
    /**
     * List of the fonts belonging to this family.
     */
    get fonts() {
        return [...new Set(this.#fontMap.values())];
    }
    get weights() {
        return [...new Set(this.fonts.map((font) => font.weight))];
    }
    get styles() {
        return [...new Set(this.fonts.map((font) => font.style))];
    }
    getFont(weightOrDesignation, style) {
        if (weightOrDesignation instanceof FontDesignation) {
            return this.#fontMap.get(weightOrDesignation);
        }
        else {
            isDefined.assert(style);
            return this.#fontMap.get(FontDesignation.for(weightOrDesignation, style));
        }
    }
    getClosestFont(weight, style) {
        const fallbacks = FontDesignation.for(weight, style).fallbacks;
        for (const fb of fallbacks) {
            if (this.#fontMap.has(fb))
                return this.#fontMap.get(fb);
        }
    }
    toString() {
        return [this.name, ...this.fallback, this.fontType]
            .map((t) => {
            const name = t instanceof FontFamily ? t.name : t;
            return name.includes(" ") ? `"${name}"` : name;
        })
            .join(", ");
    }
}

function useTypography(specNames) {
    const { typography } = useTheme();
    const validSpecName = asArray(specNames).find((name) => typography[name] != null);
    return validSpecName != null ? typography[validSpecName] : undefined;
}

export { Color, ColorConsumer, ColorPicker, ColorProvider, ColorScale, ComputedTheme, ContrastLevel, DropShadow, FixedColorScale, Font, FontDesignation, FontFamily, FontSpec, FontStyle, FontType, FontWeight, Image, Input, Rectangle, RoundedCorners, Shade, SolidBorder, SolidFill, ThemedStyle as Style, SurfaceParts, SurfaceSpec, Text, TextTransform, Theme, ThemeConsumer, ThemeProvider, ThemedStyle, UniformLightnessColorScale, Upload, View, aliceBlue, antiqueWhite, aqua, aquamarine, azure, beige, bisque, black, blanchedAlmond, blue, blueViolet, brown, burlywood, cadetBlue, chartreuse, chocolate, coral, cornflowerBlue, cornsilk, createThemeSxProcessor, crimson, cyan, darkBlue, darkCyan, darkGoldenrod, darkGray, darkGreen, darkKhaki, darkMagenta, darkOliveGreen, darkOrange, darkOrchid, darkRed, darkSalmon, darkSeaGreen, darkSlateBlue, darkSlateGray, darkTurquoise, darkViolet, deepPink, deepSkyBlue, dimGray, dodgerBlue, easeInCubic, easeInOutCubic, easeInOutQuad, easeInOutQuart, easeInOutQuint, easeInQuad, easeInQuart, easeInQuint, easeOutCubic, easeOutQuad, easeOutQuart, easeOutQuint, firebrick, floralWhite, forestGreen, fuchsia, gainsboro, getWCAGContrast, ghostWhite, gold, goldenrod, gray, green, greenYellow, hex, honeydew, hotPink, hsl, hsla, indianRed, indigo, isConvertibleToStyle, ivory, khaki, lavender, lavenderBlush, lawnGreen, lemonChiffon, lightBlue, lightCoral, lightCyan, lightGoldenrodYellow, lightGray, lightGreen, lightPink, lightSalmon, lightSeaGreen, lightSkyBlue, lightSlateGray, lightSteelBlue, lightYellow, lime, limeGreen, linear, linen, magenta, maroon, mediumAquamarine, mediumBlue, mediumOrchid, mediumPurple, mediumSeaGreen, mediumSlateBlue, mediumSpringGreen, mediumTurquoise, mediumVioletRed, midnightBlue, mintCream, mistyRose, moccasin, navajoWhite, navy, oldLace, olive, oliveDrab, orange, orangeRed, orchid, paleGoldenrod, paleGreen, paleTurquoise, paleVioletRed, papayaWhip, peachPuff, peru, pink, plum, powderBlue, purple, red, resolveThemedStyleProperties, rgb, rgba, rosyBrown, royalBlue, saddleBrown, salmon, sandyBrown, seaGreen, seashell, sienna, silver, skyBlue, slateBlue, slateGray, snow, springGreen, steelBlue, tan, tanh, teal, thistle, tomato, transparent, turquoise, u, units, useCurrentBackground, useCurrentForeground, useCurrentPalette, usePalette, useStaticStyle, useSurface, useTheme, useThemedStyle, useTypography, violet, wheat, white, whiteSmoke, yellow, yellowGreen };
